@@ -15,13 +15,24 @@ class _GlobalState:
                      max_bars=None,
                      log=None,
                      return_as_dict=None,
+                     native_python_objects=None,
                      ):
-        # self.convert_namedtuples_to_dict = False
+        """Initializes the instance variables and provides a method for setting the state with a single call.
+
+        :param debug_logging:
+        :param raise_on_errors:
+        :param max_bars:
+        :param log:
+        :param return_as_dict:
+        :param native_python_objects:
+        :return:
+        """
         self.debug_logging = debug_logging or False
         self.raise_on_errors = raise_on_errors or False
         self.max_bars = max_bars or 100_000
         self.logger = log or print
         self.return_as_dict = return_as_dict or False
+        self.native_python_objects = native_python_objects or False
 
     def get_state(self):
         state = dict(
